--[[
RabbitGear

Specialization for an rabbit gear on an mower.

Author:		Ifko[nator]
Datum:		14.08.2018

Version:	v1.0

History:	v1.0 @ 14.08.2018 - initial implementation
			-------------------------------------------------------------------------------------------------------------------------------------------------------------
		
@ xml entry: (optional)

		<rabbitGear activeSpeed="15" inActiveSpeed="7" activeSpeedLimit="8" inActiveSpeedLimit="6" animationName="setRabbitGearSwitch"/>
		
		<animation name="setRabbitGearSwitch">
            <part node="_INDEX_" startTime="0" endTime="2" startRot="0 0 0" endRot="70 0 0"/>
        </animation>
]]

RabbitGear = {};

local modFilename, _, _ = Utils.removeModDirectory(g_currentModDirectory);

function RabbitGear.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end

function RabbitGear:load(savegame)
	self.rabbitGear = {};
	self.rabbitGear.isActive = false;
	self.rabbitGear.activeSpeed = Utils.getNoNil(getXMLInt(self.xmlFile, "vehice.rabbitGear#activeSpeed"), 15) + 1; --## add 1 km/h to get the wanted speed
	self.rabbitGear.inActiveSpeed = Utils.getNoNil(getXMLInt(self.xmlFile, "vehice.rabbitGear#inActiveSpeed"), 7) + 1; --## add 1 km/h to get the wanted speed
	self.rabbitGear.activeSpeedLimit = Utils.getNoNil(getXMLInt(self.xmlFile, "vehice.rabbitGear#activeSpeedLimit"), 8) + 1; --## add 1 km/h to get the wanted speed
	self.rabbitGear.inActiveSpeedLimit = Utils.getNoNil(getXMLInt(self.xmlFile, "vehice.rabbitGear#inActiveSpeedLimit"), 6) + 1; --## add 1 km/h to get the wanted speed
	self.rabbitGear.animationName = Utils.getNoNil(getXMLString(self.xmlFile, "vehice.rabbitGear#animationName"), "setRabbitGearSwitch");
	
	if savegame ~= nil and not savegame.resetVehicles then
		self.rabbitGear.isActive = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. "#rabbitGearActive"), self.rabbitGear.isActive);
	end;
	
	self.setRabbitGear = RabbitGear.setRabbitGear;
	self:setRabbitGear(self.rabbitGear.isActive, false);
	
	self.modName = "";
	
	for _, mod in pairs(ModsUtil.modList) do
		if SpecializationUtil.getSpecialization(tostring(mod.modName) .. ".gearboxMogli") ~= nil then		
			if g_modIsLoaded[tostring(mod.modName)] then	
				self.modName = _G[tostring(mod.modName)];
			end;
		end;
	end;
	
	if self.modName ~= "" then
		self.modName.gearboxMogli.update = Utils.overwrittenFunction(self.modName.gearboxMogli.update, RabbitGear.gearBoxUpdate);
		self.modName.gearboxMogli.draw = Utils.overwrittenFunction(self.modName.gearboxMogli.draw, RabbitGear.gearBoxDraw);
	end;
end;

function RabbitGear:gearBoxUpdate(superFunc, dt)
	if self.rabbitGear ~= nil then
		return;
	else
		if superFunc ~= nil then
			return superFunc(self, dt);
		end;
	end;
end;

function RabbitGear:gearBoxDraw(superFunc)
	if self.rabbitGear ~= nil then
		return;
	else
		if superFunc ~= nil then
			return superFunc(self);
		end;
	end;
end;

function RabbitGear:update(dt)
	if self.mrGbMGetIsOn ~= nil and self:mrGbMGetIsOn() then
		--## stop gearbox addon for this vehicle
		
		self:mrGbMSetIsOnOff(false);
	end;
	
	if self:getIsMotorStarted() and self:getIsActiveForInput() then	
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
			self:setRabbitGear(not self.rabbitGear.isActive, false);
			
			--self.rabbitGear.isActive = not self.rabbitGear.isActive;
		end;
	end;
end;

function RabbitGear:updateTick(dt)
	if self.isClient then
		local maxSpeed = self.rabbitGear.inActiveSpeed;
		self.speedLimit = self.rabbitGear.inActiveSpeedLimit;
		
		if self.rabbitGear.isActive then
			maxSpeed = self.rabbitGear.activeSpeed;
			self.speedLimit = self.rabbitGear.activeSpeedLimit;
		end;
		
		if self.animations[self.rabbitGear.animationName] ~= nil then
			if self.rabbitGear.isActive then
				self:playAnimation(self.rabbitGear.animationName, 1, nil, true);
			else
				self:playAnimation(self.rabbitGear.animationName, -1, nil, true);
			end;
		end;
		
		if self.getIsTurnedOn ~= nil and self:getIsTurnedOn() then
			maxSpeed = self.speedLimit;
		end;
		
		if self.cruiseControl ~= nil and self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_ACTIVE then
			maxSpeed = math.min(maxSpeed, self.cruiseControl.speed);
		end;
		
		self.motor:setSpeedLimit(maxSpeed);
	end;
end;

function RabbitGear:draw()
	if self:getIsMotorStarted() then
		if self.rabbitGear.isActive then
			g_currentMission:addHelpButtonText(g_i18n:getText("STOP_RABBIT_GEAR"), InputBinding.IMPLEMENT_EXTRA4);
		else	
			g_currentMission:addHelpButtonText(g_i18n:getText("START_RABBIT_GEAR"), InputBinding.IMPLEMENT_EXTRA4);
		end;
	end;
end;

function RabbitGear:writeStream(streamId, connection)
    streamWriteBool(streamId, self.rabbitGear.isActive);
end;

function RabbitGear:readStream(streamId, connection)
	local isActive = streamReadBool(streamId);

    self:setRabbitGear(isActive, true, true);
end;

function RabbitGear:setRabbitGear(isActive, noEventSend)
	if isActive ~= self.rabbitGear.isActive then
		self.rabbitGear.isActive = isActive;
		
		RabbitGearIsActiveEvent.sendEvent(self, isActive, noEventSend);
	end;
end;

function RabbitGear:getSaveAttributesAndNodes(nodeIdent)
	return 'rabbitGearActive="' .. tostring(self.rabbitGear.isActive) .. '"';
end;

--## unused but needed functions

function RabbitGear:delete()end;
function RabbitGear:mouseEvent(posX, posY, isDown, isUp, button)end;
function RabbitGear:keyEvent(unicode, sym, modifier, isDown)end;

--## toggle rabbit gear on/off

RabbitGearIsActiveEvent = {};
RabbitGearIsActiveEvent_mt = Class(RabbitGearIsActiveEvent, Event);

InitEventClass(RabbitGearIsActiveEvent, "RabbitGearIsActiveEvent");

function RabbitGearIsActiveEvent:emptyNew()
	local self = Event:new(RabbitGearIsActiveEvent_mt);
    
	return self;
end;

function RabbitGearIsActiveEvent:new(mower, rabbitGearActive)
	local self = RabbitGearIsActiveEvent:emptyNew();
	
	self.mower = mower;
	self.rabbitGearActive = rabbitGearActive;
	
	return self;
end;

function RabbitGearIsActiveEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	
	self.mower = networkGetObject(id);
	self.rabbitGearActive = streamReadBool(streamId);
    self:run(connection);
end;

function RabbitGearIsActiveEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.mower));
	streamWriteBool(streamId, self.rabbitGearActive);
end;

function RabbitGearIsActiveEvent:run(connection)
	self.mower:setRabbitGear(self.rabbitGearActive, true);
	
	if not connection:getIsServer() then
		g_server:broadcastEvent(RabbitGearIsActiveEvent:new(self.mower, self.rabbitGearActive), nil, connection, self.mower);
	end;
end;

function RabbitGearIsActiveEvent.sendEvent(mower, rabbitGearActive, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(RabbitGearIsActiveEvent:new(mower, rabbitGearActive), nil, nil, mower);
		else
			g_client:getServerConnection():sendEvent(RabbitGearIsActiveEvent:new(mower, rabbitGearActive));
		end;
	end;
end;